#include <pwd.h>
#include <grp.h>
#include <stdio.h>
#include <errno.h>
#include <dirent.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include "make_path.h"
#include "get_stat.h"
#include "recurse.h"
#include "unused.h"

static char r_flag;
static char *s_flag = "chown";
static char H_flag;
static char v_flag;
static char g_flag;
static int (*chown_func)(const char *pathname, uid_t owner, gid_t group);
static long gid = -1;
static long uid = -1;

static int change(const char *path, void *p) {
	char *name = (char *)p;

	struct stat sb;
	if (mu_get_lstat(s_flag, path, &sb))
		return 1;

	long gid_priv = (gid == -1) ? sb.st_gid : gid;
	long uid_priv = (uid == -1) ? sb.st_uid : uid;
 
	if (chown_func(path, uid_priv, gid_priv)) {
		if (s_flag)
			fprintf(stderr, "chown: unable to chown %s: %s\n", path, strerror(errno));

		return 1;
	}

	if (v_flag)
		printf("chown: %s: changed to %s\n", path, name);

	return 0;
}

static void get_owner(char *arg) {
	char *group = strchr(arg, ':');
	char gg_flag = 1;
	char gu_flag = 1;

	if (group == NULL) {
		if (g_flag)
			gu_flag = 0;

		else
			gg_flag = 0;
	}

	if (gg_flag) {
		if (g_flag == 0) {
			group[0] = '\0';
			group++;
		}

		else
			group = arg;

		struct group *grp = getgrnam(group);
		if (!grp) {
			if (s_flag)
				fprintf(stderr, "chown: invalid group: %s\n", group);

			exit(1);
		}

		gid = grp->gr_gid;
	}

	if (gu_flag) {
		struct passwd *pwd = getpwnam(arg);
		if (!pwd) {
			if (s_flag)
				fprintf(stderr, "chown: invalid user: %s\n", arg);

			exit(1);
		}

		uid = pwd->pw_gid;
	}
}

int main(int argc, char **argv) {
	chown_func = lchown;

	int opt;
	while ((opt = getopt(argc, argv, "RsHvg")) != -1) {
		switch (opt) {
			case 'R':
				r_flag = 1;
				break;

			case 's':
				s_flag = NULL;
				break;

			case 'H':
				H_flag = 1;
				chown_func = chown;
				break;

			case 'v':
				v_flag = 1;
				break;

			case 'g':
				g_flag = 1;
				break;

			default:
				puts("chown [RsHvg] USER[:[GRP]]/GRP [file1 file2...]\n\t-H Symbolic link\n\t-R Recursive\n\t-s Silent\n\t-v Verbose\n\t-g Change only group");
				return 0;
		}
	}

	argv += optind;
	argc -= optind;

	if (argc == 0) {
		fprintf(stderr, "chown: missing operand\n");
		return 1;
	}

	get_owner(argv[0]);

	int ret = 0;
	for (int i = 1; i < argc; i++) {
		if (r_flag) {
			if (mu_recurse(s_flag, H_flag, argv[i], argv[0], change, change))
				ret = 1;
		}

		else
			ret = change(argv[i], argv[0]);
	}

	return ret;
}
