#define _XOPEN_SOURCE
#include <fcntl.h>
#include <stdlib.h>
#include <unistd.h>
#include <utime.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "parse_date.h"

static char c_flag;
static char m_flag;
static char a_flag;

static int setdate(const char *path, const time_t date, char flag) {
	struct utimbuf new_time;

	if (flag || a_flag)
		new_time.actime = date;

	if (flag || m_flag)
		new_time.modtime = date;

	if (utime(path, &new_time) < 0) {
		fprintf(stderr, "touch: %s: %s\n", path, strerror(errno));
		return 1;
	}

	return 0;
}

static int touch(const char *path, const time_t date) {
	if (!c_flag) {
		int fd = open(path, O_CREAT | O_RDONLY, 0666);
		if (fd < 0) {
			fprintf(stderr, "touch: %s: %s\n", path, strerror(errno));
			return 1;
		}

		close(fd);
	}

	/* Update the date to now */
	if (setdate(path, time(NULL), 1))
		return 1;

	/* Seting custom values */
	char flag = 0;
	if (m_flag == 0 && a_flag == 0)
		flag = 1;

	if (date != -1 && setdate(path, date, flag))
		return 1;

	return 0;
}

int main(int argc, char **argv) {
	time_t date = -1;

	int opt;
	while ((opt = getopt(argc, argv, "cmad:t:")) != -1) {
		switch (opt) {
			case 'm':
				m_flag = 1;
				break;

			case 'a':
				a_flag = 1;
				break;

			case 'c':
				c_flag = 1;
				break;

			case 't':
			case 'd':
				date = mu_parse_date("touch", optarg);
				if (date == -1)
					return 1;

				break;

			default:
				puts("touch [cmad] [file1 file2...]\n\t-c Don't create files\n\t-a Change only atime\n\t-m Change only mtime\n\t-d Date/time to use\n\nRecognized TIME formats:");
				for (size_t i = 0; i < sizeof(mu_date_fmts) / sizeof(char *); i++)
					puts(mu_date_fmts[i]);

				return 0;
		}
	}

	argv += optind;
	argc -= optind;

	if (argc == 0) {
		printf("touch: missing operand\n");
		return 1;
	}

	for (int i = 0; i < argc; i++)
		if (touch(argv[i], date))
			return 1;

	return 0;
}
